unit UltraWav;

interface

uses ultradrv, crt;

type wavType = record
       dlzka, pozicia:longint;
       frekvencia:longint;
       bit16:boolean;
     end;

procedure loadWav(meno:string; var wav:wavType);
procedure playWav(kanal:byte; wav:wavType);
procedure freeWav(wav:wavType);

procedure playLongWav(kanal:byte; meno:string); {zahra wav zo suboru}

implementation

const SIZE_BUF = 32768;
      SIZE_SEG = 65536;

procedure loadWav(meno:string; var wav:wavType);
var inf:file;
    Buffer:pointer;
    datasize,adresa:longint;
    nacital,citaj:word;
    pole:array[1..4] of byte;
    format:array[0..$17] of byte; {alebo len $13 ? ...}
    stereo:boolean;
    mode:byte;
begin
   GetMem(Buffer, SIZE_BUF); {pre citanie zo suborov}
   {Nahraj vsetko do pamati GUSa}
   assign(inf,meno); {vezmi parameter}
   {$I-}
   reset(inf,1);
   {$I+}
   if IOResult <> 0 then
      begin writeln('File Read error: ',meno); halt(1); end;

   blockread(inf, pole, 4); {RIFF}
   blockread(inf, pole, 4); {all data size}
   datasize := (((longint(pole[4])*256)+pole[3])*256+pole[2])*256+pole[1];

   blockread(inf, pole, 4); {WAVE}
   blockread(inf, pole, 4); {fmt }
   blockread(inf, pole, 4); {format data size}
   datasize := (((longint(pole[4])*256)+pole[3])*256+pole[2])*256+pole[1];
   blockread(inf, format, datasize); {format info}
   stereo := (format[2] = 2);
   wav.frekvencia := (word(format[5])*256)+format[4];
   if stereo then wav.frekvencia := 2*wav.frekvencia; {zdvojnasob frekvenciu}
   wav.bit16 := (format[14] = $10);
   if wav.bit16 then mode:=DMA_Data_16Bit
   else mode:=Convert_Data;

   blockread(inf, pole, 4); {data}
   blockread(inf, pole, 4); {datasize}
   datasize := (((longint(pole[4])*256)+pole[3])*256+pole[2])*256+pole[1];
   {uz je datasize aktualna dlzka dat vo wave}

   wav.dlzka := datasize;
   if wav.dlzka > 256*1024 then wav.dlzka := 256*1024;

   if UltraMemAlloc(wav.dlzka,wav.pozicia) then
   begin
      datasize := wav.dlzka; adresa:=0;
      while (datasize > 0) do
      begin
         if (datasize > SIZE_BUF) then citaj:=SIZE_BUF
         else citaj:=datasize;
         if (((wav.pozicia + adresa) div SIZE_SEG) < {ostro mensie (o 1)}
            ((wav.pozicia + adresa + citaj) div SIZE_SEG)) then
         begin
            citaj:= SIZE_SEG - (wav.pozicia + adresa) mod SIZE_SEG;
         end;
         blockread(inf, Buffer^, citaj, nacital);
         datasize := datasize - nacital;
         if not UltraDownLoad(Buffer,
                              mode,
                              wav.pozicia+adresa, {nech zacne az tu}
                              nacital,
                              true) then
         begin
            UltraMemFree(wav.dlzka,wav.pozicia);
            writeln('Error downloading to GUS.');
            halt(1);
         end;
         adresa := adresa + nacital;
      end; {while}
   end;

   {$I-}
   close(inf);
   {$I+}

   FreeMem(Buffer, SIZE_BUF);
end;

procedure playWav(kanal:byte; wav:wavType);
var mode:byte;
begin
   {vycentruj stereo}
   UltraSetBalance(kanal, 7);
   {Nastav rozumnu hlasitost}
   UltraSetLinearVolume(kanal, 400);
   {Nastav frekvenciu}
   UltraSetFrequency(kanal, wav.frekvencia);
   if wav.bit16 then mode:=Voice_data_16Bit
   else mode:=0;
   UltraStartVoice(kanal,
                   wav.pozicia,
                   wav.pozicia,
                   wav.pozicia+wav.dlzka-1-ord(wav.bit16),
                   mode);
end;

procedure freeWav(wav:wavType);
begin
   UltraMemFree(wav.dlzka,wav.pozicia);
end;

procedure playLongWav(kanal:byte; meno:string);
var inf:file;
    Buffer:pointer;
    datasize,adresa,adresa2:longint;
    nacital,citaj:word;
    pole:array[1..4] of byte;
    format:array[0..$17] of byte; {alebo len $13 ? ...}
    stereo:boolean;
    mode,playmode:byte;
    wav:wavType;
    xxx:longint;
    alokoval:byte; {0 - nic, 1 - segment, 2 - 2segmenty}
    pozicia:longint; {kde zacina novy segment v alokovanom}
begin
   GetMem(Buffer, SIZE_BUF); {pre citanie zo suborov}
   assign(inf,meno); {vezmi parameter}
   {$I-}
   reset(inf,1);
   {$I+}
   if IOResult <> 0 then
      begin writeln('File Read error: ',meno); halt(1); end;

   blockread(inf, pole, 4); {RIFF}
   blockread(inf, pole, 4); {all data size}
   datasize := (((longint(pole[4])*256)+pole[3])*256+pole[2])*256+pole[1];

   blockread(inf, pole, 4); {WAVE}
   blockread(inf, pole, 4); {fmt }
   blockread(inf, pole, 4); {format data size}
   datasize := (((longint(pole[4])*256)+pole[3])*256+pole[2])*256+pole[1];
   blockread(inf, format, datasize); {format info}
   stereo := (format[2] = 2);
   wav.frekvencia := (word(format[5])*256)+format[4];
   if stereo then wav.frekvencia := 2*wav.frekvencia; {zdvojnasob frekvenciu}
   wav.bit16 := (format[14] = $10);
   if wav.bit16 then mode:=DMA_Data_16Bit
   else mode:=Convert_Data;

   blockread(inf, pole, 4); {data}
   blockread(inf, pole, 4); {datasize}
   datasize := (((longint(pole[4])*256)+pole[3])*256+pole[2])*256+pole[1];
   {uz je datasize aktualna dlzka dat vo wave}

   wav.dlzka := datasize;

   alokoval := 0;
   if UltraMemAlloc(SIZE_SEG,wav.pozicia) then
   begin
      if ((wav.pozicia mod SIZE_SEG) <> 0) then
      begin
         UltraMemFree(SIZE_SEG,wav.pozicia);
         if UltraMemAlloc(2*SIZE_SEG,wav.pozicia) then
         begin
            pozicia:=((wav.pozicia+SIZE_SEG div SIZE_SEG) + 1) * SIZE_SEG;
            alokoval:=2;
         end else alokoval:=0;
      end else begin
         pozicia:=wav.pozicia;
         alokoval:=1;
      end;
   end;
   if alokoval > 0 then
   begin
      {predpriprava kanalu:}
      {vycentruj stereo}
      UltraSetBalance(kanal, 7);
      {Nastav rozumnu hlasitost}
      UltraSetLinearVolume(kanal, 400);
      {Nastav frekvenciu}
      UltraSetFrequency(kanal, wav.frekvencia);
      if wav.bit16 then playmode:=Voice_data_16Bit
      else playmode:=0;

      datasize := wav.dlzka;
      adresa:=0; {kolko som uz nacital...}
      adresa2:=0; {ze kam to ma davat do pamati GUSa}
      while ((datasize > 0) and (not keypressed)) do
      begin
         if (datasize > SIZE_BUF) then citaj:=SIZE_BUF
         else citaj:=datasize;
         blockread(inf, Buffer^, citaj, nacital);
         datasize := datasize - nacital;
         if not UltraDownLoad(Buffer,
                              mode,
                              pozicia+adresa2, {kam (0 alebo pol)}
                              nacital,             {dlzka}
                              true) then
         begin
            UltraMemFree(alokoval*SIZE_SEG,wav.pozicia);
            writeln('Error downloading to GUS.');
            halt(1);
         end else begin {download sa podaril}
            adresa2:= adresa2+ nacital;
            if (adresa2 >= SIZE_SEG) then adresa2:=adresa2-SIZE_SEG;
            adresa := adresa + nacital;

            if (datasize = 0) then
            begin
               if (adresa < SIZE_SEG) then
               begin
                  UltraStartVoice(kanal,
                                  pozicia, {begin}
                                  pozicia, {start}
                                  pozicia+wav.dlzka-1-ord(wav.bit16),
                                  playmode);
               end else begin
                  while (UltraReadVoice(kanal) >= pozicia+adresa2) do;
                     for xxx:=1 to 100 do {velmi mala pauza}
                        if (xxx<1) then dec(xxx); {delay}
                  UltraSetVoiceEnd(kanal,pozicia+adresa2-1-ord(wav.bit16));
                  UltraSetLoopMode(kanal, playmode); {nech neloopuje}
               end;
            end;

            if (adresa = SIZE_SEG) then {uz sa nacital komplet segment}
            begin
               {hraj s koncom v strede, s rolloverom a NOT loopingom}
               UltraStartVoice(kanal,
                               pozicia, {begin}
                               pozicia, {start}
                               pozicia+SIZE_SEG-1-ord(wav.bit16), {end}
                               playmode or Loop_Voice); {dokola}
            end;
            if (adresa >= SIZE_SEG) then
            begin {uz je vsetko v pohode - standard}
               if (adresa2 = 0) then
               begin
                  while ((UltraReadVoice(kanal)) < pozicia+SIZE_BUF) do
                     for xxx:=1 to 10000 do
                        if (xxx<1) then dec(xxx); {delay}
                  {ok, continue}
               end else begin
                  while ((UltraReadVoice(kanal)) > pozicia+SIZE_BUF) do
                     for xxx:=1 to 10000 do
                        if (xxx<1) then dec(xxx); {delay}
               end;
               {pokracuj, nahraj prvu (resp. druhu) polovicu}
            end;
         end;
      end; {while}
   end;

   if (keypressed) then
   begin
      while (keypressed) do if (readkey = ' ') then;
      UltraStopVoice(kanal);
   end;

   {$I-}
   close(inf);
   {$I+}

   UltraMemFree(alokoval*SIZE_SEG,wav.pozicia);
   FreeMem(Buffer, SIZE_BUF);

   while (not UltraVoiceStopped(kanal)) do
      for xxx:=1 to 10000 do
          if (xxx<1) then dec(xxx); {delay}
end;

end.
